	globals	on
	case	off
	chip	65c02
	pw	132
	pl	59
;V1.01
;RAW DISK DUMP UTILITY FOR C1P/UK101 running DOS/65
;
;DUMPS THE CONTENTS OF A DISK AND SENDS OUTPUT
;TO THE serial PORT AS ASCII HEX. IT WILL
;CAPTURE ALL DATA ON THE DISK INCLUDING THE
;TRACK/SECTOR FORMATTING INFORMATION.
;
;Timing of all delay loops or delay routines
;are based on use of the same routines used
;by the standard DOS/65 routines to calculate
;and use a CPU clock rate dependent timing
;parameter SPEED.
;
;DUMPS EVERY BYTE ENCOUNTERED ON A TRACK
;STARTING AT THE INDEX PULSE except V1.01
;filters out any header noise bytes while
;checking that the header contents are correct.
;
;Note that the dump format is as defined by
;the OSI tools. In particular the overall
;data structure is:
;	a. Each track is contained within a
;	nine page record.
;	b. Each record consists of the seven
;	byte header, the data track, and
;	fill to pad the total record to nine
;	pages.
;	c. Every byte including all bytes from
;	the disk and the fill bytes are encoded
;	as two hexadecimal characters followed
;	by an ASCII space.
;	d. For human readability bytes within
;	a record are grouped into sub-records
;	of 16 bytes followed by an ASCII CR and
;	LF.
;	e. Each record  is preceded with a blank
;	line consisting of just an ASCII CR and KF.
;	f. For DOS/65 versions using 5.25 inch drives
;	the data track is always 2048 bytes (8 pages)
;	long after the seven byte header. As a result
;	DOS/65 records will always have 256-7=249
;	bytes of fill after the actual track data.
;
;This DOS/65 utility is based on the OS65D utility
;for the C1P V0.4.
;Initial Release:	3 August 2015 (V1.00)
;Last Revision:		8 August 2015 (V1.01)
;				filters noise in header
;DOS/65 definitions:
pem	equ	$103		;PEM entry
tea	equ	$200		;TEA start
pgzstr	equ	2		;page zero section start
warm	equ	0		;warm boot command
conin	equ	6		;read console input command w/o echo
conout	equ	2		;send char to console output
strout	equ	9		;send $ terminated string to console
;five inch disk parameters
MAXTRK	equ	$40		;# TRACKS TO READ in BCD
MAXPAG	equ	9		;# PAGES TO READ
FILLCH	equ	0		;FILL CHARACTER
;C1P hardware definitions
;serial 6850 ACIA
serial	equ	$f001		;serial port
;disk controller related
DISK	equ	$C000		;DISK CONTROLLER base
;pia
;read only
flpsts	equ	disk		;status port
;bit definitions
; 0
; 1	track 0 if 0
; 2
; 3
; 4
; 5	write protect if 0
; 6	drive select 1 (0=B or D, 1=A or C)
; 7	index if 0
;write only
flpcnt	equ	disk+2
;bit definitions
; 0	write enable if 0
; 1
; 2	step
;	  0 if in
;	  1 if out (to track zero)
; 3	step on falling edge
; 4
; 5	drive select 2 (0=C or D, 1=A or B)
; 6
; 7	head load if 0
;acia
flpdta	equ	disk+17		;data port
;ASCII characters
CR	equ	$0D
LF	equ	$0A
	page0
;page zero section
	org	pgzstr
cnt	defs	2		;count
trk	defs	1		;track number in BCD
PagLim	defs	1		;pages in buffer
dtapag	defs	1		;number of data pages
BufPnt	defs	2		;pointer into buffer
Point	defs	2		;gp pointer
speed	defs	1		;cpu speed factor
char	defs	1		;save of character
chrnum	defs	1		;character number in page
BlkNum	defs	1		;position in 16 byte block
SaveX	defs	1		;save X in Output
DtaPnt	defs	2		;Pointer into buffer for data
	ends
	code
;code section
	ORG	tea		;start of code
	lda	#<opnmsg	;point to opening message
	ldy	#>opnmsg
	jsr	string		;print it
	JSR	ANYKEY
	jsr	serint		;initialize serial port
	JSR	INITPIA		;initialize disk PIA
	jsr	rstdsk		;initialize disk ACIA
;set timer constant SPEED - track does not matter
	lda	#255		;say is quite fast
	sta	speed
;find true cpu speed
	jsr	loadhd		;load head
ind0	lda	flpsts		;get status
	bmi	ind0		;loop until index
ind1	lda	flpsts		;get again
	bpl	ind1		;loop until end of index
	ldx	#0		;clear counters
	ldy	#0
chkrev	lda	flpsts		;get again
	bpl	gotrev		;branch if index
	nop			;delay a bit
	jmp	j0		;and then some more
j0	inx
	bne	chkrev
	iny
	jmp	chkrev
gotrev	txa			;see if should round
	bpl	b1		;branch if should not
	iny			;else bump speed
b1	sty	speed		;set speed
	jsr	unldhd
;SPEED has been calculated and set - just to gain
;some confidence send SPEED to console
	lda	#<spdmsg	;point to message
	ldy	#>spdmsg
	jsr	string		;print it
	lda	speed		;now send SPEED
	jsr	pdeca
	LDA	#0		;start at track 0
	STA	TRK
LOOP	LDA	TRK		;see if new track at max
	CMP	#MAXTRK
	BEQ	QUIT		;done if is
	lda	#<rdemsg	;say reading
	ldy	#>rdemsg
	jsr	string		;on console
	lda	trk		;get track
	jsr	PhexAC		;send to console as BCD
	lda	trk		;get track again
	BNE	LOOP2		;if track not 0 step to it
	JSR	TZERO		;else home drive TO TRACK 0
	JMP	LOOP3		;and read track
;valid track --> position head
LOOP2	JSR	TNEXT		;STEP TO NEXT TRACK
LOOP3	lda	trk		;initialize header structure
	sta	trknum		;with track number
	JSR	RDTRK		;read track into buffer
	lda	#<savmsg	;point to saving message
	ldy	#>savmsg
	jsr	string		;send it
	LDA	#MAXPAG
	JSR	SAVBUF		;send buffer to serial port
	clc			;treat track number as BCD
	sed
	lda	trk		;add 1
	adc	#1
	cld
	sta	trk		;and save
	JMP	LOOP		;and loop for next track
;done processing disk - clean up & exit
QUIT	lda	#<dnemsg	;point to message
	ldy	#>dnemsg
	jsr	string		;send it
	JSR	INPUT
	ldx	#warm
	jmp	pem
;SAVE BUFFER CONTENTS by sending to host system
;through the serial port.
;At entry A has maximum page number to save
SAVBUF	STA	PagLim		;save max page number
	JSR	RESPTR
	LDA	#0		;initialize counters
	STA	CNT
	STA	CNT+1
	sta	ChrNum		;set index to zero
	sta	BlkNum		;set block number to zero
	BEQ	SAVBUF5
SAVBUF1	lda	BlkNum		;get block position number
	BNE	SAVBUF2		;if in block skip ahead
	lda	#cr		;but if start send cr & lf
	jsr	serout
	lda	#lf
	jsr	serout
	lda	#16
	sta	BlkNum		;set block position counter
	jmp	SAVBUF3
SAVBUF2	LDA	#' '		;put space between blocks
	JSR	serout
SAVBUF3	ldy	ChrNum
	LDA	(BufPnt),Y	;get byte from buffer
	JSR	PHEXA		;send as hex
	INC	CNT		;bump count
	BNE	SAVBUF4
	INC	CNT+1
SAVBUF4	dec	BlkNum		;drop block position number
	inc	ChrNum		;bump index
	BNE	SAVBUF1		;loop if not roll over
	INC	BufPnt+1	;else bump pointer
	DEC	PagLim		;and drop page count
SAVBUF5	LDA	PagLim
	BNE	SAVBUF1		;loop if more
;done sending buffer close with cr and lf
	lda	#cr		;send cr and lf to serial
	jsr	serout
	lda	#lf
	jmp	serout
;INIT POINTERS, CLEAR BUFFER
INIT	JSR	RESPTR		;SET POINTERS
	LDY	#0
	LDA	#FILLCH
INIT1	STA	(BufPnt),Y
	INY
	BNE	INIT1
	INC	BufPnt+1
	DEC	PagLim
	BNE	INIT1
;RESET BUFFER POINTERS
RESPTR	LDA	#MAXPAG
	STA	PagLim
	lda	#maxpag-1
	sta	dtapag		;set amount of data to read
	LDA	#<BUFFER
	STA	BufPnt
	LDA	#>BUFFER
	STA	BufPnt+1
	RTS
;INIT DISK CONTROLLER PIA
INITPIA	LDY	#0
	STY	DISK+1		;SELECT DDRA
	STY	DISK		;SET PORTA TO ALL INPUTS
	LDX	#4		;0000 0100
	STX	DISK+1		;SELECT PORTA
	STY	DISK+3		;SELECT DDRB
	DEY
	STY	DISK+2		;SET PORTB TO ALL OUTPUTS
	STX	DISK+3		;SELECT PORTB
	STY	DISK+2		;SET PORTB OUTPUTS HIGH
	RTS
;STEP TO TRACK 0
TZERO	LDA	#%11111011
	BNE	TZERO3		;DIR=INWARDS
TZERO1	LDA	#%00000010
	BIT	DISK		;TEST 'TRK0' PIN
	BNE	TZERO2		;AT TRACK 0
	RTS			;there so done
;not there so move head
TZERO2	LDA	#%11111111	;DIR=OUTWARDS (TO TRK0)
TZERO3	STA	DISK+2		;SET 'DIR' PIN
	JSR	dly24		;wait 24 cycles
	AND	#%11110111	;1111 0111
	STA	DISK+2		;SET 'STEP' PIN LOW
	JSR	dly24		;wait 24 cycles
	ORA	#8		;0000 1000
	STA	DISK+2		;SET 'STEP' PIN HIGH
	LDX	#60		;60 MS
	JSR	dlyxm
	jmp	TZERO1		;ALWAYS
;STEP TO NEXT TRACK
TNEXT	LDA	#%11111011	;DIR=INWARDS
	STA	DISK+2		;SET 'DIR' PIN
	JSR	dly24		;wait 24 cycles
	AND	#%11110111	;1111 0111
	STA	DISK+2		;SET 'STEP' LOW
	JSR	dly24		;wait 24 cycles
	ORA	#%00001000	;0000 1000
	STA	DISK+2		;SET 'STEP' HIGH
	LDX	#60		;60MS
	JMP	dlyxm
;READ CURRENT TRACK TO BUFFER
RDTRK	JSR	INIT		;set pointers and clear buffer
;write header to buffer
	ldy	#0		;clear index
hdrlpe	lda	header,y	;get character
	sta	(bufpnt),y	;put in buffer
	iny			;bump index
	cpy	#7		;see if done
	bne	hdrlpe		;loop if more
;correct header has been written to buffer
;now set up data pointer
	lda	#<(buffer+7)
	ldy	#>(buffer+7)
	sta	dtapnt
	sty	dtapnt+1
	LDX	#%01111111	;0111 1111
	STX	DISK+2		;LOAD DISK HEAD
	ldx	#160		;delay total 320ms
	jsr	dlyxm
	LDX	#160
	JSR	dlyxm
RDTRKi	LDA	DISK		;WAIT START OF
	BMI	RDTRKi		;INDEX PULSE
RDTRK2	LDA	DISK		;WAIT END OF
	BPL	RDTRK2		;INDEX PULSE
	LDA	#%00000011	;RESET ACIA
	STA	DISK+$10
	LDA	#%01011000	;DIVIDE BY 1, 8E1, RTS HIGH
	STA	DISK+$10
;look for 'C' at header start
RDTRKC	JSR	RDBYT
	BCS	trkerr		;OR END-OF-TRACK
	cmp	#'C'		;look for C
	bne	rdtrkc		;loop if not
rdtrkW	jsr	rdbyt		;now look for W
	bcs	trkerr		;error if end if track
	cmp	#'W'		;check byte
	bne	rdtrkW		;loop if wrong
rdtrkn	jsr	rdbyt		;should be track number
	bcs	trkerr		;error if end of track
	cmp	trk		;see if correct
	bne	rdtrkn		;loop if wrong
rdtrkX	jsr	rdbyt		;next should be X
	bcs	trkerr		;error if EOT
	cmp	#'X'		;look for X
	bne	rdtrkX		;loop if not
rdtrkv	jsr	rdbyt		;get next
	bcs	trkerr		;error if EOT
	cmp	#'v'		;see if v
	bne	rdtrkv		;loop if not
rdtrk1	jsr	rdbyt		;read next
	bcs	trkerr		;error if EOT
	cmp	#1		;see if 1
	bne	rdtrk1		;loop if not
rdtrk8	jsr	rdbyt		;now look for 8
	bcs	trkerr		;error if EOT
	cmp	#8		;see if 8
	bne	rdtrk8		;loop if not
;At this point header has been read and checked
;now start data capture.
	ldy	#0
bytlpe	jsr	rdbyt		;get byte
	sta	(dtapnt),y	;save
	iny			;bump index
	BNE	bytlpe		;loop if page incomplete
	INC	DtaPnt+1	;else bump pointer
	DEC	DtaPag		;see if last data page
	BNE	bytlpe		;no - more to do
	LDA	#$FF
	STA	DISK+2		;UNLOAD DISK HEAD
	RTS
;There was an error reading track so abort operation
TRKERR	lda	#$ff		;unload head
	sta	disk+2
	lda	#<errmsg	;send error message
	ldy	#>errmsg
	jsr	string		;to console
	jmp	quit		;and quit
;Timing routines based on SPEED or just cycle count
;delay 100us x times
dly100	lda	speed		;get constant C4
d100lp	jmp	j1		;waste time C3
j1	sec			;C2
	sbc	#5		;drop by five C2
	bcs	d100lp		;loop if more C3 (last C2)
	dex			;drop x C2
	bne	dly100		;loop to start C3 (last C2)
	rts			;C6
;delay=20*y+14 cycles
dlyy20	jsr	dly15		;delay 15 cycles
	dey			;drop count
	bne	dlyy20		;loop if more
	nop			;waste time
	rts
;delay=15 cycles
dly15	jmp	dly12		;C3
;delay=12 cycles
dly12	rts
;delay=24 cycles
dly24	jsr	dly12		;do 12
	rts
;delay=12ms
dly12m	ldx	#12
;delay=xms
dlyxm	ldy	speed
	jsr	dlyy20		;do 20*y cycles
	dex
	bne	dlyxm		;loop if more
	rts
;load head and wait 40 ms
loadhd	lda	#%01111111	;set load bit
	and	flpcnt		;to active
	sta	flpcnt
dly40m	ldx	#41		;delay 40 ms
	bne	dlyxm
;unload head and delay
unldhd	lda	#%10000000	;set load bit
	ora	flpcnt		;to inactive
	sta	flpcnt
	bne	dly40m		;and delay
;do reset of disk acia
rstdsk	lda	#%00000011	;master reset
	sta	flpdta-1
	lda	#%01011000	;no interrupt,rts* high,8+ep+s,/1
	sta	flpdta-1
	rts
;End of SPEED based code
;INPUT CHAR FROM DISK, CY SET IF END-OF-TRACK
RDBYT	LDA	DISK		;TEST FOR INDEX PULSE
	BPL	RDBYT2
	LDA	DISK+$10
	LSR	A
	BCC	RDBYT
	BIT	DISK+$10 	;TEST PARITY ERROR
	BVS	RDBYT3
	LDA	DISK+$11 	;READ BYTE
	CLC
	RTS
;was end of track
RDBYT2	LDA	#FILLCH
	SEC
	RTS
RDBYT3
RDBYT4	LDA	DISK+$11 	;READ BYTE
	CLC
	RTS
;Print $ terminated string of up to 255 characters pointed to by
;A and Y.
string	ldx	#strout
	jmp	pem
;PRINT HEX BYTE in a to serial port
PHEXA	PHA
	LSR	A		;do high nibble first
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXA1
	PLA
PHEXA1	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXA2
	CLC
	ADC	#7
PHEXA2	JMP	serout		;send to serial port
;PRINT HEX BYTE in a to console
PHEXAC	PHA
	LSR	A		;do high nibble first
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXAC1
	PLA
PHEXAC1	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXAC2
	CLC
	ADC	#7
PHEXAC2	JMP	output		;send to console
;PRINT DECIMAL (A) to console
PDECA	TAX
	LDA	#0
;PRINT DECIMAL (A,X) to console
PDEC	STX	PDEC8
	STA	PDEC8+1
	LDA	#0
	LDX	#2
PDEC1	STA	PDEC10,X
	DEX
	BPL	PDEC1
	LDY	#16
	SED			;set to decimal mode
PDEC2	ASL	PDEC8
	ROL	PDEC8+1
	LDX	#2
PDEC3	LDA	PDEC10,X
	ADC	PDEC10,X
	STA	PDEC10,X
	DEX
	BPL	PDEC3
	DEY
	BNE	PDEC2
	cld			;set back to binary mode
	LDX	#6
	LDA	#0
	STA	PDEC9
PDEC4	LDY	#3
	LDA	#0
PDEC5	ASL	PDEC10+2
	ROL	PDEC10+1
	ROL	PDEC10
	ROL	A
	DEY
	BPL	PDEC5
	TAY
	BNE	PDEC6
	CPX	#1
	BEQ	PDEC6
	LDY	PDEC9
	BEQ	PDEC7
PDEC6	INC	PDEC9
	ORA	#$30
	JSR	OUTPUT
PDEC7	DEX	
	BNE	PDEC4
	RTS
PDEC8	WORD	0
PDEC9	BYTE	0
PDEC10	BYTE	0,0,0
;send CR & LF to console
CRLF	LDA	#CR
	JSR	OUTPUT
	LDA	#LF
;output character in A to console - save A & X
output	sta	char		;save character for serial
	stx	SaveX		;save x register
	ldx	#conout
	jsr	pem
	ldx	SaveX		;get x register back
	lda	char
	rts
;output char in a to serial port
serout	pha			;save character
serlpe	lda	serial-1	;get status
	lsr	a		;test for empty
	lsr	a
	bcc	serlpe		;loop if not
	pla
	sta	serial		;send it
	rts
;wait for any key to be pressed
ANYKEY	lda	#<anymsg	;point to message
	ldy	#>anymsg
	JSR	string		;send it
;wait for key press but do not echo
;return key in A
input	ldx	#conin		;get input w/o echo
	jmp	pem		;through DOS/65
;serial port routines
;initialize serial port to /16
serint	lda	#%00000011
	sta	serial-1
	lda	#%00010001
	sta	serial-1
	rts
;message strings
;opening splash
opnmsg	BYTE	CR,LF
	BYTE	'DOS/65 OSI disk dump V1.01',CR,LF
	BYTE	'Dumps header and data from',CR,LF
	BYTE	'disk to serial port.',CR,LF
	BYTE	'Write protect disk!',CR,LF
	byte	'Turn on serial port capture on host.'
	BYTE	CR,LF
	BYTE	'Insert disk to be dumped.$'
;SPEED message
spdmsg	byte	cr,lf
	byte	'CPU clock speed factor = $'
;press any key
anymsg	BYTE	CR,LF
	BYTE	'Press any key when ready',CR,LF,'$'
;reading track
rdemsg	BYTE	CR,LF
	BYTE	'Reading track $'
;saving track
savmsg	BYTE	' ...saving$'
;Done message
dnemsg	BYTE	CR,LF
	BYTE	'Done ... remove disk',CR,LF
	byte	'Turn off serial port capture on host.'
	byte	cr,lf,'$'
;error message
errmsg	BYTE	CR,LF,'Header errors - Aborting$'
;header structure
header	byte	'CW'
trknum	byte	0		;track number
	byte	'Xv',1,8
;uninitialized data
;full track buffer
buffer	defs	1		;256*maxpag bytes
	ends
	END
